// Bipolar Junction Transistor (BJT) Tester
//
// CONNECT BASE (B) pin to A1 (CENTER PIN)
// CONNECT OTHER TWO LEGS TO A0 AND A2
/////////////////////////////////////////////////

#include "ssd1306.h"

#define A0_680R 9
#define A0_470K 8
#define A1_680R 7
#define A1_470K 6
#define A2_680R 4
#define A2_470K 3

#define LeftButton 12

const PROGMEM uint8_t npn_bmp[] = { 
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x1E, 0x21, 0x21, 0x21,  0x12, 0x00, 0x80, 0x7F,

  0x00, 0x00, 0x00, 0x7E,  0x4A, 0x4A, 0x4A, 0x34,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0xFC, 0x40,
  0x20, 0x10, 0x08, 0x04,  0x02, 0x01, 0x00, 0x00,

  0x00, 0x01, 0x01, 0x01,  0x01, 0x01, 0x01, 0x01,
  0x01, 0x01, 0x01, 0x01,  0x01, 0x01, 0x7F, 0x04,
  0x88, 0xD0, 0xE0, 0xF0,  0xF8, 0x00, 0x00, 0x00,

  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0xFC,
  0x94, 0x94, 0x84, 0x00,  0x00, 0x01, 0x02, 0xFC
};

const PROGMEM uint8_t pnp_bmp[] = { 
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x1E, 0x21, 0x21, 0x21,  0x12, 0x00, 0x80, 0x7F,

  0x00, 0x00, 0x00, 0x7E,  0x4A, 0x4A, 0x4A, 0x34,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0xFC, 0x40,
  0x20, 0x10, 0x08, 0x04,  0x02, 0x01, 0x00, 0x00,

  0x00, 0x01, 0x01, 0x01,  0x01, 0x01, 0x01, 0x01,
  0x01, 0x01, 0x01, 0x01,  0x01, 0x01, 0x7F, 0x04,
  0xF8, 0x78, 0x38, 0x58,  0x88, 0x00, 0x00, 0x00,

  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0xFC,
  0x94, 0x94, 0x84, 0x00,  0x00, 0x01, 0x02, 0xFC
};

void setup() {
  ssd1306_setFixedFont(ssd1306xled_font6x8);
  ssd1306_128x64_i2c_init();
  ssd1306_clearScreen();
  
  pinMode(LeftButton, INPUT_PULLUP);
  Serial.begin(9600);
}

void loop() {

  float  V0TO1, V1TO0, V1TO2, V2TO1;
  int i;

  //****** V drop from A1 to A0 through 680R
  // A2 OPEN
  pinMode(A2, INPUT); 
  pinMode(A2_680R, INPUT); 
  pinMode(A2_470K, INPUT); 
  // A0 680R to GND
  pinMode(A0, INPUT);
  pinMode(A0_470K, INPUT); 
  pinMode(A0_680R, OUTPUT);   
  digitalWrite(A0_680R, 0);
  // A1 Direct to 5V
  pinMode(A1_680R, INPUT); 
  pinMode(A1_470K, INPUT);
  pinMode(A1, OUTPUT); 
  digitalWrite(A1, 1);
  // READ A0 FIFTEEN TIMES AND AVERAGE
  V1TO0 = 0;
  for (i=0;i<15;i++) {
    V1TO0 += analogRead(A0);
    delay(30);
  }
  V1TO0 /= 15;
  Serial.print("Voltage drop A1 to A0: ");
  Serial.println(V1TO0);

  //****** V drop from A0 to A1 through 680R
  // A2 OPEN
  pinMode(A2, INPUT); 
  pinMode(A2_680R, INPUT); 
  pinMode(A2_470K, INPUT); 
  // A0 Direct to 5V
  pinMode(A0_680R, INPUT); 
  pinMode(A0_470K, INPUT);
  pinMode(A0, OUTPUT); 
  digitalWrite(A0, 1);
  // A1 680R to GND
  pinMode(A1, INPUT);
  pinMode(A1_470K, INPUT); 
  pinMode(A1_680R, OUTPUT);   
  digitalWrite(A1_680R, 0);
  // READ A1 FIFTEEN TIMES AND AVERAGE
  V0TO1 = 0;
  for (i=0;i<15;i++) {
    V0TO1 += analogRead(A1);
    delay(30);
  }
  V0TO1 /= 15;
  Serial.print("Voltage drop A0 to A1: ");
  Serial.println(V0TO1);
  
  //****** V drop from A1 to A2 through 680R
  // A0 OPEN
  pinMode(A0, INPUT); 
  pinMode(A0_680R, INPUT); 
  pinMode(A0_470K, INPUT); 
  // A2 680R to GND
  pinMode(A2, INPUT);
  pinMode(A2_470K, INPUT); 
  pinMode(A2_680R, OUTPUT);   
  digitalWrite(A2_680R, 0);
  // A1 Direct to 5V
  pinMode(A1_680R, INPUT); 
  pinMode(A1_470K, INPUT);
  pinMode(A1, OUTPUT); 
  digitalWrite(A1, 1);
  // READ A2 FIFTEEN TIMES AND AVERAGE
  V1TO2 = 0;
  for (i=0;i<15;i++) {
    V1TO2 += analogRead(A2);
    delay(30);
  }
  V1TO2 /= 15;
  Serial.print("Voltage drop A1 to A2: ");
  Serial.println(V1TO2);

  //****** V drop from A2 to A1 through 680R
  // A0 OPEN
  pinMode(A0, INPUT); 
  pinMode(A0_680R, INPUT); 
  pinMode(A0_470K, INPUT); 
  // A2 Direct to 5V
  pinMode(A2_680R, INPUT); 
  pinMode(A2_470K, INPUT);
  pinMode(A2, OUTPUT); 
  digitalWrite(A2, 1);
  // A1 680R to GND
  pinMode(A1, INPUT);
  pinMode(A1_470K, INPUT); 
  pinMode(A1_680R, OUTPUT);   
  digitalWrite(A1_680R, 0);
  // READ A1 FIFTEEN TIMES AND AVERAGE
  V2TO1 = 0;
  for (i=0;i<15;i++) {
    V2TO1 += analogRead(A1);
    delay(30);
  }
  V2TO1 /= 15;
  Serial.print("Voltage drop A2 to A1: ");
  Serial.println(V2TO1);
      
  ssd1306_clearScreen( );
  
  if ((V2TO1>100) && (V0TO1>100) && (V1TO2<100) && (V1TO0<100)) {
    Serial.println(">>> PNP Bipolar Junction Transistor");
    ssd1306_printFixed(0,  0, "PNP Transistor", STYLE_NORMAL);
    ssd1306_drawBitmap(10, 10, 24, 32, pnp_bmp);
    if (V0TO1 > V2TO1)  //Evaluate Pin Order
      ssd1306_printFixed(80,  50, "CBE", STYLE_NORMAL);
    else
      ssd1306_printFixed(80,  50, "EBC", STYLE_NORMAL);
  }
  
  if ((V1TO2>100) && (V1TO0>100) && (V2TO1<100) && (V0TO1<100)) {
    Serial.println(">>> NPN Bipolar Junction Transistor");
    ssd1306_printFixed(0,  0, "NPN Transistor", STYLE_NORMAL);
    ssd1306_drawBitmap(10, 10, 24, 32, npn_bmp);
    if (V1TO0 > V1TO2)  //Evaluate Pin Order
      ssd1306_printFixed(80,  50, "CBE", STYLE_NORMAL);
    else
      ssd1306_printFixed(80,  50, "EBC", STYLE_NORMAL);
  }
    
  //Wait for Left Button to be pressed
  while(digitalRead(LeftButton) == HIGH)
    delay(50);
  delay(400);
}
